import contextlib
import sys

pytest_plugins = ("pytester",)


# Work-around known issue in pytest:
# https://github.com/pytest-dev/pytest/issues/935
# Related to pytest-cov loading after this plugin itself, producing an
# incomplete coverage report.
# Removing the plugin here ensures that it will be reloaded next time
# it's imported.
with contextlib.suppress(KeyError):
    del sys.modules["pytest_pogo"]



import os
from pathlib import Path

import asyncpg
import pytest

import pogo_migrate.config
import pytest_pogo  # noqa: F401


@pytest.fixture
def postgres_dsn():
    return os.environ["POSTGRES_DSN"]


def test_config_fixture(pogo_config):
    assert pogo_config == pogo_migrate.config.Config(
        migrations=Path.cwd() / "tests/migrations",
        database_config="{POSTGRES_DSN}",
        root_directory=Path.cwd(),
    )


@pytest.mark.usefixtures("pogo_engine")
async def test_engine_applies_migrations(pogo_config):
    db = await asyncpg.connect(pogo_config.database_dsn)
    stmt = """
    SELECT tablename
    FROM pg_tables
    WHERE  schemaname = 'public'
    ORDER BY tablename
    """
    results = await db.fetch(stmt)

    assert [r["tablename"] for r in results] == ["_pogo_migration", "_pogo_version", "table_one", "table_two"]



"""Collection of useful commands for code management.

To view a list of available commands:

$ invoke --list
"""

import subprocess

import invoke


def current_branch():
    """Get the current branch from git cli using subprocess."""
    try:
        rev_parse_out = (
            subprocess
            .check_output(
                [
                    "git",
                    "rev-parse",
                    "--tags",
                    "--abbrev-ref",
                    "HEAD",
                ],
                stderr=subprocess.STDOUT,
            )
            .decode()
            .strip()
            .split("\n")
        )
    except subprocess.CalledProcessError as e:
        msg = "Could not get current branch name."
        raise invoke.exceptions.Exit(msg) from e

    return rev_parse_out[-1]


def enforce_branch(branch_name):
    """Enforce that the current branch matches the supplied branch_name."""
    if current_branch() != branch_name:
        msg = f"Command can not be run outside of {branch_name}."
        raise invoke.exceptions.Exit(msg)


@invoke.task
def install(context):
    """Install production requirements."""
    context.run("uv sync")


@invoke.task
def install_dev(context):
    """Install development requirements."""
    context.run("uv sync --all-extras")
    context.run("uv run pre-commit install")


@invoke.task
def check_style(context):
    """Run style checks."""
    context.run("ruff check .")


@invoke.task
def tests(context):
    """Run pytest unit tests."""
    context.run("pytest -x -s")


@invoke.task
def tests_coverage(context):
    """Run pytest unit tests with coverage."""
    context.run("pytest --cov -x --cov-report=xml")


@invoke.task
def infra_test_start(context):
    """Run local unittest infrastructure."""
    context.run(
        "docker compose -f unittest-compose.yml up -d",
    )


@invoke.task
def infra_test_stop(context):
    """Stop local unittest infrastructure."""
    context.run(
        "docker compose -f unittest-compose.yml down",
    )



import logging
import sys
from contextlib import contextmanager

import pytest

import pogo_migrate.config
import pogo_migrate.testing


@contextmanager
def caplog_session(request: pytest.FixtureRequest):  # noqa: ANN201
    request.node.add_report_section = lambda *args: None  # noqa: ARG005
    logging_plugin = request.config.pluginmanager.getplugin("logging-plugin")
    for _ in logging_plugin.pytest_runtest_setup(request.node):
        yield pytest.LogCaptureFixture(request.node, _ispytest=True)


@pytest.fixture
def pogo_config() -> pogo_migrate.config.Config:
    return pogo_migrate.config.load_config()


def extract_verbosity() -> int:
    verbose = set(sys.argv) & {"-v", "-vv", "-vvv"}
    # If short verbosity is passed strip the `-` to get count of `v`s for verbosity setting.
    verbose = len(verbose.pop()) - 1 if verbose else 0

    # If full verbose flag is passed in, get count of flags
    verbose = sys.argv.count("--verbose") if "--verbose" in sys.argv else verbose

    return {
        0: logging.ERROR,
        1: logging.WARNING,
        2: logging.INFO,
        3: logging.DEBUG,
    }.get(verbose, logging.DEBUG)


@pytest.fixture(scope="session")
async def pogo_engine(request: pytest.FixtureRequest):  # noqa: ANN201
    level = extract_verbosity()
    with caplog_session(request) as caplog, caplog.at_level(level):
        await pogo_migrate.testing.apply()

    yield

    # Rollback all migrations
    with caplog_session(request) as caplog, caplog.at_level(level):
        await pogo_migrate.testing.rollback()



"""
second migration
"""

__depends__ = ["20240310_01_vlvda-initial-migration"]


async def apply(db):
    await db.execute("CREATE TABLE table_two()")


async def rollback(db):
    await db.execute("DROP TABLE table_two")
